// Livebox Decrypt.c
//
//  Written by Andrew Potter (a.k.a. - AndyP)
//  agp@uwclub.net
//
//  V1.10  8 Jul 2008 Updated to decrypt sqshfs partitions from the latest dwb files
//  V1.00 20 Nov 2005 Initial Release
//
//  This program is copyright (C) 2005 Andrew Potter (a.k.a. AndyP)
//  This program is free software; you can redistribute it and/or modify it
//  under the terms of version 2 the GNU General Public License as published
//  by the Free Software Foundation.
//  This program is distributed in the hope that it will be useful, but WITHOUT
//  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
//  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
//  more details.
//  To view a copy of the license go to:
//  http://www.fsf.org/copyleft/gpl.html
//  To receive a copy of the GNU General Public License write the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//
//  A program to decrypt the Inventel manufactured Wanadoo Livebox Recovery dwb files.
//  This dwb file contains the GNU/Linux kernel and other Open Source / GPL'd programs
//  as well as some non open source files written by Inventel and Broadcom. This was
//  hastily thrown together without a lot of thought. If anyone wants to improve it, go
//  ahead.
// /////////////////////////////////////////////////////////////////////////
// decryption for Firmware v5-01-4-uk Factory_Script from the Recovery CD
//
// 0x00000000 to 0x00000152 Header Data. Needs decoding starting at 0x00000000
//
// 0x00000000                         66697320 696e6974 202d660a 0a66636f fis init -f  fco
// 0x00000010                         6E666967 20626F6F 745F7363 72697074 nfig boot_script
// 0x00000020                         20747275 650a6663 6f6e6669 6720626f  true fconfig bo
// 0x00000030                         6f745f73 63726970 745f6461 74610a66 ot_script_data f
// 0x00000040                         6973206C 6F616420 2D622030 78383030 is load -b 0x800
// 0x00000050                         31303030 30202D6D 20637261 6D667320 10000 -m cramfs
// 0x00000060                         2d662049 6d616765 20757365 725f6673 -f Image user_fs
// 0x00000070                         0A657865 63202D63 2022626F 6F745F6C  exec -c "boot_l
// 0x00000080                         6f616465 723d5265 64426f6f 7420726f oader=RedBoot ro
// 0x00000090                         6f743d31 46303120 6d656d3d 31364d22 ot=1F01 mem=16M"
// 0x000000A0                         20307838 30303130 3030300A 0A0A6663  0x80010000   fc
// 0x000000B0                         6F6E6669 6720626F 6F745F73 63726970 onfig boot_scrip
// 0x000000C0                         745F7469 6D656F75 74203230 0a0a6669 t_timeout 20  fi
// 0x000000D0                         73206372 65617465 202D6C20 30783030 s create -l 0x00
// 0x000000E0                         30363030 3030202d 66203078 42454239 060000 -f 0xBEB9
// 0x000000F0                         30303030 202d6e20 6a666673 5f737973 0000 -n jffs_sys
// 0x00000100                         74656D0A 0A666973 20637265 61746520 tem  fis create
// 0x00000110                         2D6F2030 78313030 30202D73 20307830 -o 0x1000 -s 0x0
// 0x00000120                         30343831 30303020 2d6c2030 78303037 0481000 -l 0x007
// 0x00000130                         36303030 30202D66 20307842 45343330 60000 -f 0xBE430
// 0x00000140                         30303020 75736572 5f66730a 0a726573 000 user_fs  res
// 0x00000150                         65740a** ******** ******** ******** et
/////////////////////////////////////////////////////////////////////////
// All this appears to be the script that will be carried out after the file has been loaded
// and decrypted.
/////////////////////////////////////////////////////////////////////////
// 0x00000153 to 0x00000FFF Common Hex String - Appears to be padding to fill the remainder
// of the first 0x0001000 bytes of the file.
/////////////////////////////////////////////////////////////////////////
// 0x00000150                         ******02 0014e815 3fa6d8a2 d8147146
// Reads                              ******00 00000000 00000000 00000000
// Crypt Key                          ******02 0014e815 3fa6d8a2 d8147146
//
// 0x00000160/190/1C0/1F0~FD0         FF800938 7608E0C0 5C3C0014 634ACDF8
// Reads                              00000000 00000000 00000000 00000000
// Crypt Key                          ff800938 7608e0c0 5c3c0014 634acdf8
//
// 0X00000170/1A0/1D0/200~FE0         96C6C20D BDE7E7BB 12CF3C21 F705CF77
// Reads                              00000000 00000000 00000000 00000000
// Crypt Key                          96c6c20d bde7e7bb 12cf3c21 f705cf77
//
// 0X00000180/1B0/1E0/210~FF0         94000302 0014E815 3FA6D8A2 D8147146
// Reads                              00000000 00000000 00000000 00000000
// Crypt Key                          94000302 0014e815 3fa6d8a2 d8147146
/////////////////////////////////////////////////////////////////////////
// 0x00001000 to 0x00480FFF CRAMFS This needs to be decrypted continuing from above. The length
// of this can only be found after it has been decrypted. Therefore initial length can only
// be calculated from eof.
/////////////////////////////////////////////////////////////////////////
// 0X00001000                         BABDC410 7608A8C0 5F3C0014 634ACDF8
// Reads                              453DCD28 00004800 03000000 00000000 E=( CRAMFS=0x00480000
// Crypt Key                          ff800938 7608e0c0 5c3c0014 634acdf8
//
// 0X00001010                         D5A9AF7D CF8294C8 77AB1C73 B8488924
// Reads                              436F6D70 72657373 65642052 4F4D4653 Compressed ROMFS
// Crypt Key                          96c6c20d bde7e7bb 12cf3c21 f705cf77
//
// 0X00001020                         EB2E2C95 0014E815 BBADD8A2 B7167146
// Reads                              7F2E2F97 00000000 840B0000 6F020000
// Crypt Key                          94000302 0014e815 3fa6d8a2 d8147146
//
// 0X00001030                         BCEF6448 046D93B3 39580014 634ACDF8
// Reads                              436F6D70 72657373 65640000 00000000 Compressed
// Crypt Key                          ff800938 7608e0c0 5c3c0014 634acdf8
//
// ....... until......
//
// 0X00480FD0                         FF800938 7608E0C0 5C3C0014 634ACDF8
// Reads                              00000000 00000000 00000000 00000000 nuls
// Crypt Key                          ff800938 7608e0c0 5c3c0014 634acdf8
//
// 0X00480FE0                         96C6C20D BDE7E7BB 12CF3C21 F705CF77
// Reads                              00000000 00000000 00000000 00000000 nuls
// Crypt Key                          96c6c20d bde7e7bb 12cf3c21 f705cf77
//
// 0X00480FF0                         94000302 0014E815 3FA6D8A2 D8147146
// Reads                              00000000 00000000 00000000 00000000 nuls
// Crypt Key                          94000302 0014e815 3fa6d8a2 d8147146
/////////////////////////////////////////////////////////////////////////
// 0x00481000 to 0x00481037 Tail Data. Don't know what its for. Possibly csum
// 0x00481038 to 0x00481FFF Common Hex String - Appears to be just a filler pattern
// to fill remainder of the 0x00001000 bytes.
/////////////////////////////////////////////////////////////////////////
// 0X00481000                         FF800908 E208E3C2 5C28771A A1E12A0A
// Reads                              00000030 94000302 0014770E CA2BE7F2
// Crypt Key                          ff800938 7608e0c0 5c3c0014 634acdf8
//
// 0X00481010                         6A68C7EB 0B1E6B85 1F53D99B 5ADACF63
// Reads                              FCAE05E6 B6F98C3E 0D9CE5BA ADDF0014
// Crypt Key                          96c6c20d bde7e7bb 12cf3c21 f705cf77
//
// 0X00481020                         65AEFE85 1D373F8E 5FC664B2 478959DB
// Reads                              F1AEFD87 1D23D79B 6060BC10 9F9D289D
// Crypt Key                          94000302 0014e815 3fa6d8a2 d8147146
//
// 0X00481030                         429F2FF9 6E8D3A6B ******** ********
// Reads                              DB1F26C1 1885DAAB
// Crypt Key                          ff800938 7608e0c0 ******** ********
//
// 0X00481030                         ******** ******** A3C3FFEB 9CB53207
// Reads                                                FFFFFFFF FFFFFFFF
// Crypt Key                          ******** ******** 5c3c0014 634acdf8
//
// 0X00481040~481FA0~481FD0           69393DF2 42181844 ED30C3DE 08FA3088
// Reads                              FFFFFFFF FFFFFFFF FFFFFFFF FFFFFFFF
// Crypt Key                          96c6c20d bde7e7bb 12cf3c21 f705cf77
//
// 0X00481050~481FB0~481FE0           6BFFFCFD FFEB17EA C059275D 27EB8EB9
// Reads                              FFFFFFFF FFFFFFFF FFFFFFFF FFFFFFFF
// Crypt Key                          94000302 0014e815 3fa6d8a2 d8147146
//
// 0X00481060~481FC0~481FF0           007FF6C7 89F71F3F A3C3FFEB 9CB53207
// Reads                              FFFFFFFF FFFFFFFF FFFFFFFF FFFFFFFF
// Crypt Key                          ff800938 7608e0c0 5c3c0014 634acdf8
//
/////////////////////////////////////////////////////////////////////////
// 0x00482000 to 0x00482037 Cryptkey Data
/////////////////////////////////////////////////////////////////////////
//                                          30=48 Bytes. Length of crypt
//                                             Repeating
//                                             ******** ******** ********
// 0X00482000                         00000030 94000302 0014E815 3FA6D8A2
//                                    ******** ******** ******** ********
// 0x00482010                         D8147146 FF800938 7608E0C0 5C3C0014
//                                    ******** ******** ******** ********
// 0x00482020                         634ACDF8 96C6C20D BDE7E7BB 12CF3C21
//                                    ******** ???????? No Idea
// 0x00482030                         F705CF77 1885DAAB
//
// Total File Size=0x00482038 = 4726840 bytes
/////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////
// Looks very similar for Firmware v5-00-RC25-fr
/////////////////////////////////////////////////////////////////////////
// 0x00000000 to 0x00000152 Header Data. Same as v5.01.4-uk firmware except for 0x00000123
/////////////////////////////////////////////////////////////////////////
// 0x00000000                         66697320 696e6974 202d660a 0a66636f fis init -f  fco
// 0x00000010                         6E666967 20626F6F 745F7363 72697074 nfig boot_script
// 0x00000020                         20747275 650a6663 6f6e6669 6720626f  true fconfig bo
// 0x00000030                         6f745f73 63726970 745f6461 74610a66 ot_script_data f
// 0x00000040                         6973206C 6F616420 2D622030 78383030 is load -b 0x800
// 0x00000050                         31303030 30202D6D 20637261 6D667320 10000 -m cramfs
// 0x00000060                         2d662049 6d616765 20757365 725f6673 -f Image user_fs
// 0x00000070                         0A657865 63202D63 2022626F 6F745F6C  exec -c "boot_l
// 0x00000080                         6f616465 723d5265 64426f6f 7420726f oader=RedBoot ro
// 0x00000090                         6f743d31 46303120 6d656d3d 31364d22 ot=1F01 mem=16M"
// 0x000000A0                         20307838 30303130 3030300A 0A0A6663  0x80010000   fc
// 0x000000B0                         6F6E6669 6720626F 6F745F73 63726970 onfig boot_scrip
// 0x000000C0                         745F7469 6D656F75 74203230 0a0a6669 t_timeout 20  fi
// 0x000000D0                         73206372 65617465 202D6C20 30783030 s create -l 0x00
// 0x000000E0                         30363030 3030202d 66203078 42454239 060000 -f 0xBEB9
// 0x000000F0                         30303030 202d6e20 6a666673 5f737973 0000 -n jffs_sys
// 0x00000100                         74656D0A 0A666973 20637265 61746520 tem  fis create
// 0x00000110                         2D6F2030 78313030 30202D73 20307830 -o 0x1000 -s 0x0
// 0x00000120                         30343836 30303020 2d6c2030 78303037 0486000 -l 0x007
// 0x00000130                         36303030 30202D66 20307842 45343330 60000 -f 0xBE430
// 0x00000140                         30303020 75736572 5f66730a 0a726573 000 user_fs  res
// 0x00000150                         65740a** ******** ******** ******** et
//
// 0x00000150                         F1740902 0014A74E 0CEE7760 F33C420F
// Reads                              ******00 00000000 00000000 00000000 nul
// Crypt Key                          94000302 0014A74E 0CEE7760 F33C420F
//
// 0x00000160/190~FD0                 B7BBA3CC 6D5FD719 18A80014 B343DB2F
// Reads                              00000000 00000000 00000000 00000000 nul
// Crypt Key                          B7BBA3CC 6D5FD719 18A80014 B343DB2F
//
// 0x00000170/1A0~FE0                 C6CFF85A A88201DA 8EEE33DB DF4B078D
// Reads                              00000000 00000000 00000000 00000000 nul
// Crypt Key                          C6CFF85A A88201DA 8EEE33DB DF4B078D
//
// 0x00000180/1B0~FF0                 94000302 0014A74E 0CEE7760 F33C420F
// Reads                              00000000 00000000 00000000 00000000 nul
// Crypt Key                          94000302 0014A74E 0CEE7760 F33C420F
/////////////////////////////////////////////////////////////////////////
// End of first 0x00001000 bytes of script.
/////////////////////////////////////////////////////////////////////////
// Now the start of the CRAMFS
/////////////////////////////////////////////////////////////////////////
// 0x00001000                         F2866EE4 6D0F9F19 1BA80014 B343DB2F
// Reads                              453DCD28 00504800 03000000 00000000 E=( CRAMFS=0x00485000
// Crypt Key                          B7BBA3CC 6D5FD719 18A80014 B343DB2F
//
// 0x00001010                         85A0952A DAE772A9 EB8A1389 900641DE
// Reads                              436F6D70 72657373 65642052 4F4D4653 Compressed ROMFS
// Crypt Key                          C6CFF85A A88201DA 8EEE33DB DF4B078D
//
// 0x00001020                         A95C6F85 0014A74E A5E57760 713E420F
// Reads
// Crypt Key                          94000302 0014A74E 0CEE7760 F33C420F
//
// 0x00001030                         F4D4CEBC 1F3AA46A 7DCC0014 B343DB2F
// Reads                              436F6D70 72657373 65640000 00000000 Compressed
// Crypt Key                          B7BBA3CC 6D5FD719 18A80014 B343DB2F
//
// 0x00001040                         2B8EF85A 208301DA 4EEA33DB 22CA078D
// Reads                              ED410000 88010000 C0040000 FD810000
// Crypt Key                          C6CFF85A A88201DA 8EEE33DB DF4B078D
//
// ....... until......
//
// 0x00485FD0                         94003020 0014A74E 0CEE7760 F33C420F
// Reads                              00003322 00000000 00000000 00000000
// Crypt Key                          94000302 0014A74E 0CEE7760 F33C420F
//
// 0x00485FE0                         B7BBA3CC 6D5FD719 18A80014 B343DB2F
// Reads                              00000000 00000000 00000000 00000000
// Crypt Key                          B7BBA3CC 6D5FD719 18A80014 B343DB2F
//
// 0x00485FF0                         C6CFF85A A88201DA 8EEE33DB DF4B078D
// Reads                              00000000 00000000 00000000 00000000
// Crypt Key                          C6CFF85A A88201DA 8EEE33DB DF4B078D
/////////////////////////////////////////////////////////////////////////
// End of CRAMFS
/////////////////////////////////////////////////////////////////////////
// Start Of Tail Data. Some of this data is the same as v5.01.4-uk but
// other parts are totally different. Not sure what it is for. Is is
// asm language ??
/////////////////////////////////////////////////////////////////////////
// 0x00486000                         94000332 9414A44C 0CFA455B 272AEFEF
// Reads                              00000030 94000302 0014323B D416ADE0
// Crypt Key                          94000302 0014A74E 0CEE7760 F33C420F
//
// 0x00486010                         1CB8BA20 5500C06D 062ABD52 F0B9DB3B
// Reads                              AB0319EC 385F1774 1E82BD46 43FA0014
// Crypt Key                          B7BBA3CC 6D5FD719 18A80014 B343DB2F
//
// 0x00486020                         7B25061F 011F9711 9FDA9F3F 870F6B18
// Reads                              BDEAFE45 A99D96CB 1134ACE4 58446C95
// Crypt Key                          C6CFF85A A88201DA 8EEE33DB DF4B078D
//
// 0x00486030                         17A45A2E 18917DE5 ******** ********
// Reads                              83A4592C 1885DAAB ******** ********
// Crypt Key                          94000302 0014A74E ******** ********
/////////////////////////////////////////////////////////////////////////
// Start of filler Data.
/////////////////////////////////////////////////////////////////////////
// 0x00486030                         ******** ******** F311889F 0CC3BDF0
// Reads                              ******** ******** FFFFFFFF FFFFFFFF
// Crypt Key                          ******** ******** 0CEE7760 F33C420F
//
// 0x00486040                         48445C33 92A028E6 E757FFEB 4CBD24D0
// Reads                              FFFFFFFF FFFFFFFF FFFFFFFF FFFFFFFF
// Crypt Key                          B7BBA3CC 6D5FD719 18A80014 B343DB2F
//
// 0x00486050                         393007A5 577DFE25 7111CC24 20B4F872
// Reads
// Crypt Key                          C6CFF85A A88201DA 8EEE33DB DF4B078D
//
// 0x00486060                         6BFFFCFD FFEB58B1 F311889F 0CC3BDF0
// Reads                              FFFFFFFF FFFFFFFF FFFFFFFF FFFFFFFF
// Crypt Key                          94000302 0014A74E 0CEE7760 F33C420F
//
// 0x00486070                         48445C33 92A028E6 E757FFEB 4CBC24D0
// Reads                              FFFFFFFF FFFFFFFF FFFFFFFF FFFFFFFF
// Crypt Key                          B7BBA3CC 6D5FD719 18A80014 B343DB2F
//
// 0x00486080                         393007A5 577DFE25 7111CC24 20B4F872
// Reads                              FFFFFFFF FFFFFFFF FFFFFFFF FFFFFFFF
// Crypt Key                          C6CFF85A A88201DA 8EEE33DB DF4B078D
//
// ...until.......
//
// 0x00486FD0                         48445C33 92A028E6 E757FFEB 4CBC24D0
// Reads                              FFFFFFFF FFFFFFFF FFFFFFFF FFFFFFFF
// Crypt Key                          B7BBA3CC 6D5FD719 18A80014 B343DB2F
//
// 0x00486FE0                         393007A5 577DFE25 7111CC24 20B4F872
// Reads                              FFFFFFFF FFFFFFFF FFFFFFFF FFFFFFFF
// Crypt Key                          C6CFF85A A88201DA 8EEE33DB DF4B078D
//
// 0x00486FF0                         6BFFFCFD FFEB58B1 F311889F 0CC3BDF0
// Reads                              FFFFFFFF FFFFFFFF FFFFFFFF FFFFFFFF
// Crypt Key                          94000302 0014A74E 0CEE7760 F33C420F
/////////////////////////////////////////////////////////////////////////
// End of Tail Data.
/////////////////////////////////////////////////////////////////////////
// Start of crypt key
/////////////////////////////////////////////////////////////////////////
//                                          30=48 Bytes. Length of crypt
//                                             Repeating pattern
// 0x00487000                         00000030 94000302 0014A74E 0CEE7760
// 0x00487010                         F33C420F B7BBA3CC 6D5FD719 18A80014
// 0x00487020                         B343DB2F C6CFF85A A88201DA 8EEE33DB
// 0x00487030                         DF4B078D 1885DAAB
//                                             ******** csum??
/////////////////////////////////////////////////////////////////////////
// Decoded Tail Data Still not sure what this is for.
// 5.01.4-uk at Offset 0X00481000
//       30 Bytes follow, decrypt with key
// 00000030 94000302 0014770E C2ABE7F2 FCAE05E6 B6F98C3E 0D9CE5BA ADDF0014 F1AEFD87 1D23D79B 6060BC10 9F9D289D DB1F26C1 1885DAAB
// 00000030 94000302 0014E815 3FA6D8A2 D8147146 FF800938 7608E0C0 5C3C0014 634ACDF8 96C6C20D BDE7E7BB 12CF3C21 F705CF77 1885DAAB
// 00000000 00000000 00009F1B FD0D3F50 24BA74A0 49798506 7B94057A F1E30000 92E4307F 8BE51596 DD875BAB 8D5214BC 4A1AE9B6 00000000
//
// 5.00.25-fr at Offset 0x00486000
// 00000030 94000302 0014323B D416ADE0 AB0319EC 385F1774 1E82BD46 43FA0014 BDEAFE45 A99D96CB 1134ACE4 58446C95 83A4592C 1885DAAB
// 00000030 94000302 0014A74E 0CEE7760 F33C420F B7BBA3CC 6D5FD719 18A80014 B343DB2F C6CFF85A A88201DA 8EEE33DB DF4B078D 1885DAAB
// 00000000 00000000 00009575 D8F8DA80 583F5BE3 8FE4B4B8 13DD6A5F 5B520000 0EA9256A 6F526E91 B9B6AD3E D6AA5F4E 5CEF5EA1 00000000
//
// Some data bits are the same. Why ? What is this for ? Could it be data for RedBoot ?
/////////////////////////////////////////////////////////////////////////
// Layout of Encrypted DWB file
///////////////////////////////
// Description          5.01.4-uk                   5.00.25-fr
// Script               0x00000000 to 0x00000152    0x00000000 to 0x00000152 Script to run after Firmware copy
// Filler               0x00000153 to 0x00000FFF    0x00000153 to 0x00000FFF Common Hex String - just a filler pattern
// CRAMFS               0x00001000 to 0x00480FFF    0x00001000 to 0x00485FFF CRAMFS
// Tail Data            0x00481000 to 0x00481003    0x00486000 to 0x00486003 Length of Tail Info 000030
// Tail Data            0x00481004 to 0x00481033    0x00486004 to 0x00486033 Tail Info crypt key ??
// Tail Data CSUM       0x00481034 to 0x00481037    0x00486034 to 0x00486037 Tail Info csum ??   1885DAAB
// Filler               0x00481038 to 0x00481FFF    0x00486038 to 0x00486FFF Common Hex String - just a filler pattern
// Crypt Data Length    0x00482000 to 0x00482003    0x00487000 to 0x00487003 Length of crypt key 00000030
// Crypt Data           0x00482004 to 0x00482033    0x00487004 to 0x00487033 Crypt key
// Checksum(?)          0x00482034 to 0x00482037    0x00487034 to 0x00487037 4 Byte Checksum ??  1885DAAB
//
// Total Size           0x00482038 = 4726840 bytes  0x00487038 = 4747320 bytes
//
/////////////////////////////////////////////////////////////////////////
//
#include "stdlib.h"
#include "stdio.h"
#include "stdint.h"
#include "assert.h"
#include "netinet/in.h"
#include "string.h"

int main(int argv, char **argc)
{
    unsigned long sourcefilelength;			// Total Length of File
    unsigned long scriptfiledataoffset;			// Offset to script file
    unsigned long cryptkeylength;			// Length of de/cryption key
    unsigned long cryptkeyoffset;			// Crypt Key Data Offset
    unsigned long cramfsoffset;				// CRAMFS Offset
    unsigned long sqshfsoffset;				// SQSHFS Offset
    unsigned long taildataoffset;			// Offset to data after CRAMFS
    unsigned char lebyte0,lebyte1,lebyte2,lebyte3;
    unsigned char *cryptkey;
    unsigned char *csum;
    unsigned long csumoffset;				// Checksum Offset

    char encrypted_script[100];
    char decrypted_script[100];
    char decrypted_ascii_script[100];
    char encrypted_cramfs[100];
    char decrypted_cramfs[100];
    char encrypted_sqshfs[100];
    char decrypted_sqshfs[100];
    char encrypted_tail[100];
    char decrypted_tail[100];
    char decrypted_tail_code[100];
    char decrypt_key[100];
    char decrypt_csum[100];
    char decrypt_keylen[100];
    char en_script[]=".script.encrypted";
    char de_script[]=".script.decrypted";
    char de_ascii_script[]=".script.ascii";
    char en_cramfs[]=".cramfs.encrypted";
    char de_cramfs[]=".cramfs.decrypted";
    char en_sqshfs[]=".sqshfs.encrypted";
    char de_sqshfs[]=".sqshfs.decrypted";
    char en_tail[]=".tail.encrypted";
    char de_tail[]=".tail.decrypted";
    char de_tail_code[]=".tail.code";
    char de_key[]=".cryptkey";
    char de_csum[]=".csum";
    char de_cryptkeylen[]=".cryptkey.length";
    int x,y;
    unsigned char z;
    int csumlength=4;
    unsigned char *inchar;
    unsigned char outchar;
    int scriptfilelength;
    int sqshfsfilecount=0;
    char squash_header[4];

    scriptfiledataoffset=0x0000;
    scriptfilelength=0x1000;
    cramfsoffset=scriptfilelength;
    for(x=0;x<argv;x++)
    {
//	printf("passed parameter %d => %s\n",x,argc[x]);
    }
    if (argv != 2)
    {
	printf("parameters -> %s infile\n",argc[0]);
	printf("infile = file to Decode.\n");
        exit(1);
    }
    sprintf(encrypted_script,"%s%s",argc[1],en_script);
    sprintf(decrypted_script,"%s%s",argc[1],de_script);
    sprintf(decrypted_ascii_script,"%s%s",argc[1],de_ascii_script);
    sprintf(encrypted_cramfs,"%s%s",argc[1],en_cramfs);
    sprintf(decrypted_cramfs,"%s%s",argc[1],de_cramfs);
    sprintf(encrypted_sqshfs,"%s%s",argc[1],en_sqshfs);
    sprintf(decrypted_sqshfs,"%s%s",argc[1],de_sqshfs);
    sprintf(encrypted_tail,"%s%s",argc[1],en_tail);
    sprintf(decrypted_tail,"%s%s",argc[1],de_tail);
    sprintf(decrypted_tail_code,"%s%s",argc[1],de_tail_code);
    sprintf(decrypt_key,"%s%s",argc[1],de_key);
    sprintf(decrypt_csum,"%s%s",argc[1],de_csum);
    sprintf(decrypt_keylen,"%s%s",argc[1],de_cryptkeylen);

    printf("%s, V1.10 Program to decode Inventel Livebox (DV-4210) Recovery DWB Files. (c) 2008 Andy Potter\n",argc[0]);
    printf("File to Decode                => %s\n",argc[1]);
    printf("Encrypted Script File         => %s\n",encrypted_script);
    printf("Decrypted Script File         => %s\n",decrypted_script);
    printf("Decrypted ASCII Script        => %s\n",decrypted_ascii_script);
    printf("Encrypted CRAMFS File         => %s\n",encrypted_cramfs);
    printf("Decrypted CRAMFS File         => %s\n",decrypted_cramfs);
    printf("Encrypted SQSHFS File         => %s\n",encrypted_sqshfs);
    printf("Decrypted SQSHFS File(s)      => %s\n",decrypted_sqshfs);
    printf("Encrypted Tail File           => %s\n",encrypted_tail);
    printf("Decrypted Tail File           => %s\n",decrypted_tail);
    printf("Decrypted Tail Code File      => %s\n",decrypted_tail_code);
    printf("Crypt Key File to Write       => %s\n",decrypt_key);
    printf("CSUM file to Write            => %s\n",decrypt_csum);
    printf("Crypt Key Length File         => %s\n",decrypt_keylen);
/* Open the encrypted dwb file */
    FILE *fp;
    if ((fp=fopen(argc[1],"rb")) == NULL)
    {
	printf("\n\nCannot open %s.\n",argc[1]);
	return 1;
    }
    fseek(fp,0,SEEK_END);
    sourcefilelength=ftell(fp);
    rewind(fp);
/* Print the size of the file and a couple of other bits */
    printf("\n");
    printf("Source File Size              => 0x%08X %d bytes\n",sourcefilelength,sourcefilelength);
    printf("Script File Data Offset       => 0x%08X %d\n",scriptfiledataoffset,scriptfiledataoffset);
    printf("CRAMFS/SQSHFS Data Offset     => 0x%08X %d\n",cramfsoffset,cramfsoffset);
    cryptkeyoffset=(sourcefilelength/0x1000)* 0x1000;
    taildataoffset=cryptkeyoffset-0x1000;
    printf("Tail Data Starts at           => 0x%08X %d\n",taildataoffset,taildataoffset);
    printf("CryptKey Data Starts at       => 0x%08X %d\n",cryptkeyoffset,cryptkeyoffset);
/* Read the cryption key length */
    fseek(fp,cryptkeyoffset,SEEK_SET);
    fread(&lebyte0,1,1,fp);
    fread(&lebyte1,1,1,fp);
    fread(&lebyte2,1,1,fp);
    fread(&lebyte3,1,1,fp);
/* and now save it to file */
    FILE *of;
    /* Crypt KeyLength File */
    if ((of=fopen(decrypt_keylen,"wb")) == NULL)
    {
	printf("\n\nCannot create %s file.\n",decrypt_keylen);
	exit(1);
    }
    fprintf(of,"%c%c%c%c",lebyte0,lebyte1,lebyte2,lebyte3);
/* Close the file */
    fclose(of);
/* Determine the encryption key length` */
/* I hate endianess */
    cryptkeylength=(((((lebyte0*256)+lebyte1)*256)+lebyte2)*256)+lebyte3;
    printf("CryptKey Length               => 0x%08X %ld bytes\n",cryptkeylength,cryptkeylength);
    cryptkey = malloc(cryptkeylength);
    if(!cryptkey)
    {
	printf("Memory Request for cryptkey failed.\n");
	exit(1);
    }
/* Read the key and store it */
    fread(cryptkey,1,cryptkeylength,fp);
    printf("CryptKey                      => ");
/* Write out the encryption/decryption key */
    /* Crypt Key File */
    if ((of=fopen(decrypt_key,"wb")) == NULL)
    {
	printf("\n\nCannot create %s file.\n",decrypt_key);
	exit(1);
    }
    fwrite(cryptkey,1,cryptkeylength,of);
/* Close the file */
    fclose(of);
/* Display the key */
    for(x=0;x<cryptkeylength;x++)
    {
	printf("%02X",*(cryptkey+x));
    }
    printf("\n");
/* Get the Checksum ?? */
    csum = malloc(4);
    if(!csum)
    {
	printf("Memory Request for csum failed.\n");
	exit(1);
    }
    if ((of=fopen(decrypt_csum,"wb")) == NULL)
    {
	printf("\n\nCannot create %s file.\n",decrypt_csum);
	exit(1);
    }
    fread(csum,1,csumlength,fp);
/* and display it and write it to file*/
    printf("CSUM ??                       => ");
    for(x=0;x<csumlength;x++)
    {
	printf("%02X",*(csum+x));
	fprintf(of,"%c",*(csum+x));
    }
    printf("\n");
    fclose(of);
/* Write out the script, encrypted and decrypted. */
    FILE *en_s, *de_s, *de_a;
    if ((en_s=fopen(encrypted_script,"wb")) == NULL)
    {
	printf("\n\nCannot create %s file.\n",encrypted_script);
	exit(1);
    }
    if ((de_s=fopen(decrypted_script,"wb")) == NULL)
    {
	printf("\n\nCannot create %s file.\n",decrypted_script);
	exit(1);
    }
    if ((de_a=fopen(decrypted_ascii_script,"wb")) == NULL)
    {
	printf("\n\nCannot create %s file.\n",decrypted_ascii_script);
	exit(1);
    }
    inchar = malloc(1);
    if(!inchar)
    {
	printf("\n\nMemory Request for inchar Failed.\n");
	exit(1);
    }
/* Seek to start of file */
    fseek(fp,0,SEEK_SET);
    printf("Writing %s, Please wait.\nWriting %s, Please wait.\n",encrypted_script,decrypted_script);
    for (x=0;x<scriptfilelength;x++)
    {
	/* include all the padding as well as the script. Might change this later. */
	fread(inchar,1,1,fp);
	fwrite(inchar,1,1,en_s);
	outchar=*inchar;
	outchar=outchar^cryptkey[x%cryptkeylength];//*inchar;
	fprintf(de_s,"%c",outchar);
	if (outchar != 0)
	{
	    fprintf(de_a,"%c",outchar);
	}
    }
    fclose(en_s);
    fclose(de_s);
    fclose(de_a);
// Now for some really iffy code

/* CRAMFS Write */
    if ((en_s=fopen(encrypted_cramfs,"wb")) == NULL)
    {
	printf("\n\nCannot create %s file.\n",encrypted_cramfs);
	exit(1);
    }
    if ((de_s=fopen(decrypted_cramfs,"wb")) == NULL)
    {
	printf("\n\nCannot create %s file.\n",decrypted_cramfs);
	exit(1);
    }
/* Seek to start of CRAMFS (end of script) */
    fseek(fp,cramfsoffset,SEEK_SET);
    printf("Writing %s, Please wait.\nWriting %s, Please wait.\n",encrypted_cramfs,decrypted_cramfs);
    for (x=cramfsoffset;x<taildataoffset;x++)
    {
	fread(inchar,1,1,fp);
	fwrite(inchar,1,1,en_s);
	outchar=*inchar;
	outchar=outchar^cryptkey[x%cryptkeylength];
	squash_header[0]=squash_header[1];
	squash_header[1]=squash_header[2];
	squash_header[2]=squash_header[3];
	squash_header[3]=outchar;
	if ((squash_header[0]=='s') && \
           (squash_header[1]=='q') && \
	   (squash_header[2]=='s') && \
	   (squash_header[3]=='h'))
	{
	    sqshfsfilecount++;
	    printf("Found sqshfs header, Closing %s\n",decrypted_cramfs);
	    fclose(de_s);
	    sprintf(decrypted_cramfs,"%s.%03X",decrypted_sqshfs,sqshfsfilecount);
	    printf("Writing %s, Please wait.\n",decrypted_cramfs);
	    if ((de_s=fopen(decrypted_cramfs,"wb")) == NULL)
	    {
		printf("\n\nCannot create %s file.\n",decrypted_cramfs);
		exit(1);
	    }
	fprintf(de_s,"s");
	fprintf(de_s,"q");
	fprintf(de_s,"s");
	//fprintf(de_s,"h");
	}
	fprintf(de_s,"%c",outchar);
    }
    fclose(en_s);
    fclose(de_s);
/* Seek to start of tail data (end of CRAMFS) */
    if ((en_s=fopen(encrypted_tail,"wb")) == NULL)
    {
	printf("\n\nCannot create %s file.\n",encrypted_tail);
	exit(1);
    }
    if ((de_s=fopen(decrypted_tail,"wb")) == NULL)
    {
	printf("\n\nCannot create %s file.\n",decrypted_tail);
	exit(1);
    }
    if ((de_a=fopen(decrypted_tail_code,"wb")) == NULL)
    {
	printf("\n\nCannot create %s file.\n",decrypted_tail_code);
	exit(1);
    }
    fseek(fp,taildataoffset,SEEK_SET);
    printf("Writing %s, Please wait\nWriting %s. Please wait.\n",encrypted_tail,decrypted_tail);
    for (x=taildataoffset;x<cryptkeyoffset;x++)
    {
	/* include all the padding as well as the tail data. Might change this later. */
	fread(inchar,1,1,fp);
	fwrite(inchar,1,1,en_s);
	outchar=*inchar;
	outchar=outchar^cryptkey[x%cryptkeylength];
	fprintf(de_s,"%c",outchar);
	if (outchar != 255)
	{
	    fprintf(de_a,"%c",outchar);
	}
    }
    fclose(en_s);
    fclose(de_s);
    fclose(de_a);
/* Close script files */
    free(csum);
    free(cryptkey);
    fclose(fp);
    return 1;
}
